@@ -11,10 +11,13 @@ from .error_handler import (
     log_error,
     log_info,
     log_warning,
+    re_raise_chained,
     safe_operation,
     set_default_handler,
+    setup_catchery_logging,
 )
 from .validation import (
+    ensure_enum,
     ensure_int_in_range,
     ensure_list_of_type,
     ensure_non_negative_int,
@@ -29,20 +32,23 @@ __all__ = [
     "AppError",
     "ErrorHandler",
     "ErrorSeverity",
+    "get_default_handler",
+    "log_critical",
+    "log_error",
     "log_info",
     "log_warning",
-    "log_error",
-    "log_critical",
+    "re_raise_chained",
     "safe_operation",
     "set_default_handler",
-    "get_default_handler",
+    "setup_catchery_logging",
     # Validation functions.
-    "validate_object",
-    "validate_type",
-    "ensure_object",
-    "ensure_string",
-    "ensure_non_negative_int",
+    "ensure_enum",
     "ensure_int_in_range",
     "ensure_list_of_type",
+    "ensure_non_negative_int",
+    "ensure_object",
+    "ensure_string",
     "safe_get_attribute",
+    "validate_object",
+    "validate_type",
 ]
