@@ -6,13 +6,18 @@ Centralized error handling and logging system.
 # Imports & Type Aliases
 # =============================================================================
 
+import atexit
+import copy  # New import for deepcopy
+import datetime
 import json
 import logging
 import threading
 from collections import deque
 from dataclasses import dataclass
 from enum import Enum
-from typing import Any, Callable, Deque, Dict, List, Protocol, TypeVar
+from functools import wraps
+from io import TextIOWrapper
+from typing import Any, Callable, Deque, Dict, List, Protocol, Type, TypeVar
 
 T = TypeVar("T")
 
@@ -45,23 +50,40 @@ class AppError:
     context: dict[str, Any]
     exception: Exception | None = None
 
+    def __post_init__(self):
+        # Ensure context is always a deep copy to prevent external modification
+        self.context = copy.deepcopy(self.context)
+
+
+class ChainedReRaiseError(RuntimeError):
+    """
+    A custom exception type used by `re_raise_chained` to indicate that an
+    exception has been re-raised as part of a chain.
+    """
+
+    pass
+
 
 class ErrorCallback(Protocol):
     def __call__(self, error: AppError) -> None: ...
 
 
 class JsonFormatter(logging.Formatter):
-    """Custom JSON formatter for logging records."""
+    """
+    Custom JSON formatter for logging records.
+    """
 
     def format(self, record: logging.LogRecord) -> str:
         """
         Formats a log record as a JSON string.
 
         Args:
-            record: The log record to format.
+            record (logging.LogRecord):
+                The log record to format.
 
         Returns:
-            A JSON string representation of the log record.
+            str:
+                A JSON string representation of the log record.
         """
         log_record: dict[str, Any] = {
             "timestamp": self.formatTime(record, self.datefmt),
@@ -80,23 +102,23 @@ class JsonFormatter(logging.Formatter):
             if key not in log_record and not key.startswith("_"):
                 log_record[key] = value
         # Ensure all values in log_record are JSON serializable
-        serializable_log_record = self._safe_json_serialize(log_record)
+        serializable_log_record = ErrorHandler._safe_json_serialize(log_record)
         return json.dumps(serializable_log_record)
 
-    def _safe_json_serialize(self, obj: Any) -> Any:
-        """
-        Recursively converts non-JSON-serializable objects within a structure
-        to their string representation.
-        """
-        if isinstance(obj, dict):
-            return {k: self._safe_json_serialize(v) for k, v in obj.items()}
-        elif isinstance(obj, (list, tuple)):
-            return [self._safe_json_serialize(elem) for elem in obj]
-        try:
-            json.dumps(obj)
-            return obj
-        except (TypeError, OverflowError):
-            return repr(obj)
+
+class ValidationWarningFilter(logging.Filter):
+    def __init__(self, handler_instance: "ErrorHandler"):
+        super().__init__()
+        self.handler_instance = handler_instance
+
+    def filter(self, record: logging.LogRecord) -> bool:
+        if (
+            self.handler_instance._suppress_validation_warnings
+            and getattr(record, "_from_validation_function", False)
+            and record.levelno == logging.WARNING
+        ):
+            return False  # Suppress the warning
+        return True  # Allow other messages
 
 
 class ErrorHandler:
@@ -115,12 +137,43 @@ class ErrorHandler:
 
     _thread_local = threading.local()
 
+    @staticmethod
+    def _safe_json_serialize(obj: Any) -> Any:
+        """
+        Recursively converts non-JSON-serializable objects within a structure to
+        their string representation.
+
+        Args:
+            obj (Any):
+                The object to serialize.
+
+        Returns:
+            Any:
+                A JSON-serializable representation of the object.
+        """
+        if isinstance(obj, dict):
+            return {k: ErrorHandler._safe_json_serialize(v) for k, v in obj.items()}
+        elif isinstance(obj, (list, tuple)):
+            return [ErrorHandler._safe_json_serialize(elem) for elem in obj]
+        elif isinstance(obj, (datetime.datetime, datetime.date, datetime.time)):
+            return obj.isoformat()
+        elif isinstance(obj, Enum):
+            return obj.value
+        elif isinstance(obj, Exception):
+            return str(obj)
+        try:
+            json.dumps(obj)
+            return obj
+        except (TypeError, OverflowError):
+            return repr(obj)
+
     def _get_thread_context(self) -> dict[str, Any]:
         """
         Retrieves the merged thread-local context from the stack.
 
         Returns:
-            A dictionary containing the merged thread-local context.
+            dict[str, Any]:
+                A dictionary containing the merged thread-local context.
         """
         if not hasattr(self._thread_local, "context_stack"):
             self._thread_local.context_stack = []
@@ -145,8 +198,8 @@ class ErrorHandler:
             Initializes the context manager with the given context data.
 
             Args:
-                **context: Arbitrary keyword arguments representing the context
-                data.
+                **context (Any):
+                    Arbitrary keyword arguments representing the context data.
             """
             self.context = context
 
@@ -163,15 +216,19 @@ class ErrorHandler:
             Exits the runtime context, popping the context from the stack.
 
             Args:
-                exc_type: The exception type, if an exception was raised.
-                exc_val: The exception value, if an exception was raised.
-                exc_tb: The traceback, if an exception was raised.
+                exc_type (Any):
+                    The exception type, if an exception was raised.
+                exc_val (Any):
+                    The exception value, if an exception was raised.
+                exc_tb (Any):
+                    The traceback, if an exception was raised.
             """
             if hasattr(ErrorHandler._thread_local, "context_stack"):
                 if ErrorHandler._thread_local.context_stack:
                     ErrorHandler._thread_local.context_stack.pop()
                 else:
-                    # This case should ideally not happen if enter/exit are balanced
+                    # This case should ideally not happen if enter/exit are
+                    # balanced
                     pass
             else:
                 # This case should ideally not happen if enter/exit are balanced
@@ -192,7 +249,8 @@ class ErrorHandler:
             Initializes the error capture context manager.
 
             Args:
-                handler: The ErrorHandler instance to capture errors from.
+                handler (ErrorHandler):
+                    The ErrorHandler instance to capture errors from.
             """
             self.handler: ErrorHandler = handler
             self.captured: List[AppError] = []
@@ -202,7 +260,8 @@ class ErrorHandler:
             Callback function to append captured errors to the list.
 
             Args:
-                error: The AppError instance to capture.
+                error (AppError):
+                    The AppError instance to capture.
             """
             self.captured.append(error)
 
@@ -211,7 +270,8 @@ class ErrorHandler:
             Enters the runtime context, registering the capture callback.
 
             Returns:
-                A list that will contain the captured AppError instances.
+                List[AppError]:
+                    A list that will contain the captured AppError instances.
             """
             self.handler.register_callback(self._callback)
             return self.captured
@@ -221,12 +281,16 @@ class ErrorHandler:
             Exits the runtime context, restoring original callbacks.
 
             Args:
-                exc_type: The exception type, if an exception was raised.
-                exc_val: The exception value, if an exception was raised.
-                exc_tb: The traceback, if an exception was raised.
+                exc_type (Any):
+                    The exception type, if an exception was raised.
+                exc_val (Any):
+                    The exception value, if an exception was raised.
+                exc_tb (Any):
+                    The traceback, if an exception was raised.
             """
             # Remove the capture callback. This assumes it's still in the list.
-            # If the list was modified externally, this might not work as expected.
+            # If the list was modified externally, this might not work as
+            # expected.
             try:
                 self.handler._callbacks.remove(self._callback)
             except ValueError:
@@ -239,42 +303,83 @@ class ErrorHandler:
         error_history_maxlen: int = 1000,
         use_json_logging: bool = False,
         plain_text_formatter: logging.Formatter | None = None,
+        text_log_path: str | None = None,
+        json_log_path: str | None = None,
+        suppress_validation_warnings: bool = True,
     ) -> None:
         """
         Initialize the ErrorHandler.
 
         Args:
-            logger: Optional custom logger instance. error_history_maxlen: Max
-            number of errors to keep in history. use_json_logging: If True, logs
-            in JSON format.
+            logger (logging.Logger | None):
+                Optional custom logger instance.
+            error_history_maxlen (int):
+                Max number of errors to keep in history.
+            use_json_logging (bool):
+                If True, logs in JSON format.
+            plain_text_formatter (logging.Formatter | None):
+                Optional custom formatter for plain text logging.
+            text_log_path (str | None):
+                Optional path to a file for persistent logging.
+            json_log_path (str | None):
+                Optional path to a JSON Lines file for storing structured
+                AppError objects.
         """
         self.error_history: Deque[AppError] = deque(maxlen=error_history_maxlen)
         self._lock = threading.Lock()
         self._callbacks: List[ErrorCallback] = []
         self._use_json_logging = use_json_logging
-        self._stored_handlers: List[logging.Handler] = []
         self._plain_text_formatter = plain_text_formatter or logging.Formatter(
             "[%(asctime)s] %(levelname)-8s: %(message)s", datefmt="%H:%M:%S"
         )
+        self._text_log_file: logging.FileHandler | None = None
+        self._json_log_file: TextIOWrapper | None = None
+        self._suppress_validation_warnings = suppress_validation_warnings
 
         if logger is None:
             self.logger = logging.getLogger("ErrorHandler")
-            handler = logging.StreamHandler()
-            self.logger.addHandler(handler)
+            if not self.logger.handlers:
+                stream_handler = logging.StreamHandler()
+                self.logger.addHandler(stream_handler)
         else:
             self.logger = logger
 
+        if text_log_path:
+            try:
+                self._text_log_file = logging.FileHandler(
+                    text_log_path, encoding="utf-8"
+                )
+                self.logger.addHandler(self._text_log_file)
+            except Exception as e:
+                self.logger.error(
+                    f"Failed to set up file logging to {text_log_path}: {e}"
+                )
+
+        if json_log_path:
+            try:
+                self._json_log_file = open(json_log_path, "a", encoding="utf-8")
+            except IOError as e:
+                self.logger.error(
+                    f"Failed to open JSON error log file {json_log_path}: {e}"
+                )
+
         if self._use_json_logging:
             self.set_json_logging()
         else:
             self.set_plain_text_logging()
 
+        # Apply the validation warning filter to all handlers
+        self._validation_warning_filter = ValidationWarningFilter(self)
+        for handler in self.logger.handlers:
+            handler.addFilter(self._validation_warning_filter)
+
     def get_logger(self) -> logging.Logger:
         """
         Returns the logger instance used by the ErrorHandler.
 
         Returns:
-            logging.Logger: The logger instance.
+            logging.Logger:
+                The logger instance.
         """
         return self.logger
 
@@ -283,8 +388,8 @@ class ErrorHandler:
         Sets a custom plain text formatter for the logger.
 
         Args:
-            formatter: The logging.Formatter instance to use for plain text
-            logging.
+            formatter (logging.Formatter):
+                The logging.Formatter instance to use for plain text logging.
         """
         self._plain_text_formatter = formatter
         if not self._use_json_logging:
@@ -309,13 +414,30 @@ class ErrorHandler:
         Sets the given formatter to all handlers of the logger.
 
         Args:
-            formatter: The logging.Formatter instance to set.
+            formatter (logging.Formatter):
+                The logging.Formatter instance to set.
         """
         for handler in self.logger.handlers:
             handler.setFormatter(formatter)
+        if self._text_log_file:
+            self._text_log_file.setFormatter(formatter)
+
+    def shutdown(self) -> None:
+        """
+        Shuts down the error handler, closing any open file handlers.
+        """
+        if self._text_log_file:
+            self.logger.removeHandler(self._text_log_file)
+            self._text_log_file.close()
+            self._text_log_file = None
+        if self._json_log_file:
+            self._json_log_file.close()
+            self._json_log_file = None
 
     def register_callback(self, callback: ErrorCallback) -> None:
-        """Register a callback to be called on every error."""
+        """
+        Register a callback to be called on every error.
+        """
         self._callbacks.append(callback)
 
     def handle(
@@ -337,13 +459,21 @@ class ErrorHandler:
             ...)
 
         Args:
-            error: Either an AppError instance or the error error string.
-            severity: The severity level of the error (required if not passing
-                      AppError).
-            context: An optional dictionary of contextual data. exception: An
-            optional exception object associated with the error.
-            raise_exception: If True, re-raises the `exception` after handling.
-            chain_exception: An optional exception to chain with `exception`.
+            error (AppError | str):
+                Either an AppError instance or the error error string.
+            severity (ErrorSeverity | None):
+                The severity level of the error (required if not passing
+                AppError).
+            context (Dict[str, Any] | None):
+                An optional dictionary of contextual data.
+            exception (Exception | None):
+                An optional exception object associated with the error.
+            raise_exception (bool):
+                If True, re-raises the `exception` after handling.
+            chain_exception (Exception | None):
+                An optional exception to chain with `exception`.
+            stack_offset (int):
+                The number of stack frames to skip when logging the error.
         """
         if not isinstance(error, AppError):
             if severity is None:
@@ -368,6 +498,29 @@ class ErrorHandler:
                 # Prevent a faulty callback from disrupting error handling
                 self.logger.exception("Error in error handler callback")
 
+        # Write structured error to JSON log file if enabled
+        if self._json_log_file:
+            try:
+                # Prepare the error data for JSON logging.
+                error_data = {
+                    "timestamp": datetime.datetime.now().isoformat(),
+                    "message": error.message,
+                    "severity": error.severity.value,
+                    "context": ErrorHandler._safe_json_serialize(error.context),
+                    "exception": str(error.exception) if error.exception else None,
+                }
+                # Write the structured error to the JSON log file.
+                self._json_log_file.write(json.dumps(error_data) + "\n")
+                # Ensure it's written to disk immediately.
+                self._json_log_file.flush()
+            except Exception as e:
+                self.logger.error(
+                    f"Failed to write structured error to JSON log file: {e}",
+                    exc_info=True,
+                )
+
+
+
         # Determine the appropriate logging method based on severity.
         log_method = {
             ErrorSeverity.LOW: self.logger.info,
@@ -430,21 +583,25 @@ class ErrorHandler:
         `handle` method.
 
         Args:
-            operation: A callable (function or lambda) representing the
-            operation
-                       to execute. It should take no arguments and return a
-                       value of type T.
-            default: The default value to return if an exception occurs during
-                     the operation.
-            error_message: A descriptive message for the error, used in logging.
-            severity: The severity level of the error if one occurs (default:
-            MEDIUM). context: An optional dictionary of additional context to
-            include
-                     with the error log.
+            operation (Callable[[], T]):
+                A callable (function or lambda) representing the operation to
+                execute. It should take no arguments and return a value of type
+                T.
+            default (T):
+                The default value to return if an exception occurs during the
+                operation.
+            error_message (str):
+                A descriptive message for the error, used in logging.
+            severity (ErrorSeverity | None):
+                The severity level of the error if one occurs (default: MEDIUM).
+            context (Dict[str, Any] | None):
+                An optional dictionary of additional context to include with the
+                error log.
 
         Returns:
-            The result of the `operation` if successful, or the `default` value
-            if an exception occurs.
+            T:
+                The result of the `operation` if successful, or the `default`
+                value if an exception occurs.
         """
         try:
             return operation()
@@ -465,28 +622,85 @@ _default_global_error_handler: ErrorHandler | None = None
 
 def get_default_handler() -> ErrorHandler:
     """
-    Returns the default global ErrorHandler instance.
-    Initializes it if it hasn't been initialized yet.
+    Returns the default global ErrorHandler instance. Initializes it if it
+    hasn't been initialized yet.
     """
     global _default_global_error_handler
     if _default_global_error_handler is None:
         _default_global_error_handler = ErrorHandler()
-        # Ensure the default logger has at least one handler if it's newly created
+        # Ensure the default logger has at least one handler if it's newly
+        # created
         if not _default_global_error_handler.logger.handlers:
             handler = logging.StreamHandler()
             _default_global_error_handler.logger.addHandler(handler)
+        atexit.register(_default_global_error_handler.shutdown)  # Register shutdown
     return _default_global_error_handler
 
 
 def set_default_handler(handler: ErrorHandler | None) -> None:
     """
-    Sets the global ErrorHandler instance.
-    This is primarily for testing or advanced configuration.
+    Sets the global ErrorHandler instance. This is primarily for testing or
+    advanced configuration.
     """
     global _default_global_error_handler
     _default_global_error_handler = handler
 
 
+def setup_catchery_logging(
+    level: int = logging.INFO,
+    text_log_path: str | None = None,
+    json_log_path: str | None = None,
+    use_json_logging: bool = False,
+    error_history_maxlen: int = 1000,
+    suppress_validation_warnings: bool = True,
+) -> ErrorHandler:
+    """
+    Sets up and returns a new default ErrorHandler instance with common logging
+    configurations.
+
+    This function simplifies the initialization of the ErrorHandler by providing
+    a convenient way to configure logging levels, file outputs, and other common
+    settings.
+
+    Args:
+        level (int):
+            The logging level for the handler (e.g., logging.INFO,
+            logging.DEBUG). Defaults to logging.INFO.
+        text_log_path (str | None):
+            Optional path to a file for plain text logging.
+        json_log_path (str | None):
+            Optional path to a JSON Lines file for storing structured AppError
+            objects.
+        use_json_logging (bool):
+            If True, the main logger will use JSON formatting. Defaults to
+            False.
+        error_history_maxlen (int):
+            Maximum number of errors to keep in history. Defaults to 1000.
+        suppress_validation_warnings (bool):
+            If True, validation warnings will be suppressed. Defaults to True.
+
+    Returns:
+        ErrorHandler:
+            The newly configured default ErrorHandler instance.
+
+    Example:
+        >>> from catchery.error_handler import setup_catchery_logging, log_error
+        >>> handler = setup_catchery_logging(
+            level=logging.DEBUG, text_log_path="app.log")
+        >>> log_error("Something went wrong!")
+    """
+    handler = ErrorHandler(
+        error_history_maxlen=error_history_maxlen,
+        use_json_logging=use_json_logging,
+        text_log_path=text_log_path,
+        json_log_path=json_log_path,
+        suppress_validation_warnings=suppress_validation_warnings,
+    )
+    handler.get_logger().setLevel(level)
+    set_default_handler(handler)
+    return handler
+
+
 # ==============================================================================
 # VALIDATION HELPERS
 # ==============================================================================
@@ -501,12 +715,16 @@ def safe_operation(
     Decorator for safe operation execution.
 
     Args:
-        default_value (Any): Default value to return on error. error_message
-        (str): Error message prefix for logging. severity (ErrorSeverity):
-        Severity level for errors.
+        default_value (Any):
+            Default value to return on error.
+        error_message (str):
+            Error message prefix for logging.
+        severity (ErrorSeverity):
+            Severity level for errors.
 
     Returns:
-        Callable: The decorator function.
+        Callable[[Callable[..., Any]], Callable[..., Any]]:
+            The decorator function.
     """
 
     def decorator(func: Callable[..., Any]) -> Callable[..., Any]:
@@ -524,6 +742,89 @@ def safe_operation(
     return decorator
 
 
+def re_raise_chained(
+    message: str,
+    new_exception_type: Type[Exception] = ChainedReRaiseError,
+    severity: ErrorSeverity = ErrorSeverity.HIGH,
+    context: Dict[str, Any] | Callable[..., Dict[str, Any]] | None = None,
+) -> Callable[[Callable[..., Any]], Callable[..., Any]]:
+    """
+    A decorator that catches exceptions from the decorated function, logs them,
+    and then re-raises a new, specified exception, chaining it to the original.
+
+    Args:
+        message (str):
+            The message for the new exception.
+        new_exception_type (Type[Exception]):
+            The type of exception to re-raise.
+        severity (ErrorSeverity):
+            The severity level for logging the original error.
+        context (Dict[str, Any] | Callable[..., Dict[str, Any]] | None):
+            Additional context for logging. Can be a dictionary or a callable
+            that takes the decorated function's args/kwargs and returns a dict.
+
+    Returns:
+        Callable[[Callable[..., Any]], Callable[..., Any]]:
+            The decorator function.
+    """
+
+    def decorator(func: Callable[..., Any]) -> Callable[..., Any]:
+        @wraps(func)
+        def wrapper(*args: Any, **kwargs: Any) -> Any:
+            handler = get_default_handler()
+            runtime_context: Dict[str, Any] = {}
+            if callable(context):
+                try:
+                    # Pass args and kwargs to the context
+                    runtime_context = context(*args, **kwargs)
+                except Exception as e:
+                    # Log if the context itself fails
+                    handler.handle(
+                        error=f"Failed to build context for '{func.__name__}': {e}",
+                        severity=ErrorSeverity.HIGH,
+                        context={
+                            "function_name": func.__name__,
+                            "error_in_context_builder": str(e),
+                        },
+                    )
+            elif context is not None:
+                runtime_context = context
+
+            try:
+                return func(*args, **kwargs)
+            except Exception as original_exception:
+                # Conditionally log the original exception if it's not a
+                # ChainedReRaiseError.
+                if not isinstance(original_exception, ChainedReRaiseError):
+                    _msg = f"{type(original_exception).__name__} "
+                    _msg += f"({func.__name__}): "
+                    _msg += f"{original_exception}"
+                    handler.handle(
+                        error=_msg,
+                        severity=severity,
+                        context=runtime_context,
+                        exception=original_exception,
+                        raise_exception=False,
+                    )
+                # Log the new exception being raised.
+                _msg = f"{new_exception_type.__name__}"
+                _msg += f" ({func.__name__}): {message}"
+                handler.handle(
+                    error=_msg,
+                    severity=severity,
+                    context=runtime_context,
+                    exception=new_exception_type(message),
+                    raise_exception=False,
+                )
+                # Re-raise the new exception, explicitly chaining it to the
+                # original.
+                raise new_exception_type(message) from original_exception
+
+        return wrapper
+
+    return decorator
+
+
 # ==============================================================================
 # CONVENIENCE LOGGING FUNCTIONS
 # ==============================================================================
@@ -540,11 +841,17 @@ def log_info(
     Logs an info-level message using the global error handler.
 
     Args:
-        message: The primary message describing the informational event.
-        context: An optional dictionary of additional context to include with
-        the log. exception: An optional exception object to include with the
-        log. raise_exception: If True, re-raises the `exception` after handling.
-        chain_exception: An optional exception to chain with `exception`.
+        message (str):
+            The primary message describing the informational event.
+        context (Dict[str, Any] | None):
+            An optional dictionary of additional context to include with the
+            log.
+        exception (Exception | None):
+            An optional exception object to include with the log.
+        raise_exception (bool):
+            If True, re-raises the `exception` after handling.
+        chain_exception (Exception | None):
+            An optional exception to chain with `exception`.
     """
     get_default_handler().handle(
         message,
@@ -568,11 +875,17 @@ def log_warning(
     Logs a warning-level message using the global error handler.
 
     Args:
-        message: The primary message describing the warning event. context: An
-        optional dictionary of additional context to include with the log.
-        exception: An optional exception object to include with the log.
-        raise_exception: If True, re-raises the `exception` after handling.
-        chain_exception: An optional exception to chain with `exception`.
+        message (str):
+            The primary message describing the warning event.
+        context (Dict[str, Any] | None):
+            An optional dictionary of additional context to include with the
+            log.
+        exception (Exception | None):
+            An optional exception object to include with the log.
+        raise_exception (bool):
+            If True, re-raises the `exception` after handling.
+        chain_exception (Exception | None):
+            An optional exception to chain with `exception`.
     """
     get_default_handler().handle(
         message,
@@ -596,11 +909,17 @@ def log_error(
     Logs an error-level message using the global error handler.
 
     Args:
-        message: The primary message describing the error event. context: An
-        optional dictionary of additional context to include with the log.
-        exception: An optional exception object to include with the log.
-        raise_exception: If True, re-raises the `exception` after handling.
-        chain_exception: An optional exception to chain with `exception`.
+        message (str):
+            The primary message describing the error event.
+        context (Dict[str, Any] | None):
+            An optional dictionary of additional context to include with the
+            log.
+        exception (Exception | None):
+            An optional exception object to include with the log.
+        raise_exception (bool):
+            If True, re-raises the `exception` after handling.
+        chain_exception (Exception | None):
+            An optional exception to chain with `exception`.
     """
     get_default_handler().handle(
         message,
@@ -624,11 +943,17 @@ def log_critical(
     Logs a critical-level message using the global error handler.
 
     Args:
-        message: The primary message describing the critical event. context: An
-        optional dictionary of additional context to include with the log.
-        exception: An optional exception object to include with the log.
-        raise_exception: If True, re-raises the `exception` after handling.
-        chain_exception: An optional exception to chain with `exception`.
+        message (str):
+            The primary message describing the critical event.
+        context (Dict[str, Any] | None):
+            An optional dictionary of additional context to include with the
+            log.
+        exception (Exception | None):
+            An optional exception object to include with the log.
+        raise_exception (bool):
+            If True, re-raises the `exception` after handling.
+        chain_exception (Exception | None):
+            An optional exception to chain with `exception`.
     """
     get_default_handler().handle(
         message,
