@@ -2,7 +2,8 @@
 This module contains support functions for validating data.
 """
 
-from typing import Any, Callable, Dict, Union
+from enum import Enum
+from typing import Any, Callable, Dict, Type, Union
 
 from .error_handler import ErrorSeverity, get_default_handler, log_warning
 
@@ -31,83 +32,77 @@ def _get_type_display_name(obj: Any) -> str:
         return type(obj).__name__
 
 
-def _attempt_default_conversion(
-    obj: Any,
-    expected_type: Any,
-    name: str,
-    ctx: dict[str, Any],
-) -> Any | None:
-    """
-    Attempts to perform a default conversion for common types (str, int, float).
-    Logs a warning if conversion fails.
-
-    Args:
-        obj (Any): The object to convert.
-        expected_type (Any): The expected type to convert to.
-        name (str): The name of the object, used for logging.
-        ctx (dict[str, Any]): The context for logging.
-
-    Returns:
-        (Any | None): The converted object or None if conversion failed.
-    """
-    try:
-        if expected_type is str and not isinstance(obj, str):
-            return str(obj)
-        elif expected_type is int and not isinstance(obj, int):
-            return int(obj)
-        elif expected_type is float and not isinstance(obj, float):
-            return float(obj)
-        elif expected_type is bool and not isinstance(obj, bool):
-            return bool(obj)
-    except (ValueError, TypeError) as e:
-        log_warning(
-            f"Default conversion of '{name}' "
-            f"to {_get_type_display_name(expected_type)} "
-            f"failed: {e}. Using default.",
-            {**ctx, "error_details": str(e)},
-        )
-    return None
-
-
-def _attempt_conversion(
+def _attempt_conversion_with_fallback(
     obj: Any,
     name: str,
     expected_type: Any,
-    expected_type_name: str,
-    converter: Callable[[Any], Any],
     ctx: dict[str, Any],
+    converter: Callable[[Any], Any] | None = None,
 ) -> Any | None:
     """
-    Attempts to convert an object using a custom converter, handling errors and
-    type mismatches.
+    Attempts to convert an object using a custom converter if provided,
+    otherwise attempts default conversions for common types.
+    Logs a warning if conversion fails or results in a type mismatch.
 
     Args:
         obj (Any): The object to convert.
         name (str): The name of the object, used for logging.
-        expected_type (Any): The expected type to convert to.
-        expected_type_name (str): The human-readable name of the expected type.
-        converter (Callable[[Any], Any]): The conversion function to apply.
+        expected_type (Any): The expected type(s) to convert to.
         ctx (dict[str, Any]): The context for logging.
+        converter (Callable[[Any], Any] | None): An optional custom conversion
+            function to apply.
 
     Returns:
         (Any | None): The converted object or None if conversion failed.
     """
-    try:
-        processed_object = converter(obj)
-        if isinstance(processed_object, expected_type):
-            return processed_object
-        log_warning(
-            f"Converter for '{name}' returned wrong type. "
-            f"Expected {expected_type_name}, "
-            f"got {_get_type_display_name(type(processed_object))}. "
-            f"Using default.",
-            ctx,
-        )
-    except Exception as e:
-        log_warning(
-            f"Conversion of '{name}' failed: {e}. Using default.",
-            {**ctx, "error_details": str(e)},
-        )
+    expected_type_name = _get_type_display_name(expected_type)
+
+    # Try custom converter first
+    if converter:
+        try:
+            processed_object = converter(obj)
+            if isinstance(processed_object, expected_type):
+                return processed_object
+            log_warning(
+                f"Converter for '{name}' returned wrong type. "
+                f"Expected {expected_type_name}, "
+                f"got {_get_type_display_name(type(processed_object))}. "
+                f"Using default.",
+                {**ctx, "_from_validation_function": True},
+            )
+            return None
+        except Exception as e:
+            log_warning(
+                f"Custom conversion of '{name}' failed: {e}. Using default.",
+                {**ctx, "error_details": str(e), "_from_validation_function": True},
+            )
+            return None
+
+    # Fallback to default conversions
+    target_types = (
+        expected_type if isinstance(expected_type, tuple) else (expected_type,)
+    )
+
+    for t in target_types:
+        try:
+            if t is str and not isinstance(obj, str):
+                return str(obj)
+            elif t is int and not isinstance(obj, int):
+                return int(obj)
+            elif t is float and not isinstance(obj, float):
+                return float(obj)
+            elif t is bool and not isinstance(obj, bool):
+                return bool(obj)
+            # If obj is already of the current target type, return it
+            if isinstance(obj, t):
+                return obj
+        except (ValueError, TypeError) as e:
+            log_warning(
+                f"Default conversion of '{name}' "
+                f"to {_get_type_display_name(t)} "
+                f"failed: {e}. Using default.",
+                {**ctx, "error_details": str(e), "_from_validation_function": True},
+            )
     return None
 
 
@@ -330,7 +325,7 @@ def ensure_object(
         log_warning(
             f"'{name}' is None and not allowed for expected type(s) "
             f"{expected_type_name}. Using default.",
-            ctx,
+            {**ctx, "_from_validation_function": True},
         )
         return default
 
@@ -339,43 +334,16 @@ def ensure_object(
         log_warning(
             f"'{name}' is not of expected type(s) {expected_type_name}. "
             f"Attempting conversion.",
-            ctx,
+            {**ctx, "_from_validation_function": True},
         )
 
-        # If we have a converter, let's try to use it.
-        if converter:
-            processed_object = _attempt_conversion(
-                obj,
-                name,
-                expected_type,
-                expected_type_name,
-                converter,
-                ctx,
-            )
-
-        elif isinstance(expected_type, tuple):
-            # Try converting to one of the types in the tuple.
-            for t in expected_type:
-                converted_attempt = _attempt_default_conversion(
-                    obj,
-                    t,
-                    name,
-                    ctx,
-                )
-                if converted_attempt is not None:
-                    processed_object = converted_attempt
-                    break
-            else:  # No break, conversion failed for all types in tuple
-                processed_object = None
-
-        else:
-            # For single expected_type.
-            processed_object = _attempt_default_conversion(
-                obj,
-                expected_type,
-                name,
-                ctx,
-            )
+        processed_object = _attempt_conversion_with_fallback(
+            obj,
+            name,
+            expected_type,
+            ctx,
+            converter,
+        )
 
     # If we failed to process the object (i.e., it's None after conversion
     # attempts), return the default value.
@@ -388,20 +356,79 @@ def ensure_object(
             if not validator(processed_object):
                 log_warning(
                     f"'{name}' failed validation. Using default.",
-                    ctx,
+                    {**ctx, "_from_validation_function": True},
                 )
                 return default
         except Exception as e:
             log_warning(
                 f"Validator for '{name}' raised an exception: {e}. "
                 f"Using default object.",
-                {**ctx, "error_details": str(e)},
+                {**ctx, "error_details": str(e), "_from_validation_function": True},
             )
             return default
 
     return processed_object
 
 
+def ensure_enum(
+    obj: Any,
+    name: str,
+    enum_class: Type[Enum],
+    default: Enum | None = None,
+    context: Dict[str, Any] | None = None,
+) -> Enum | None:
+    """
+    Ensures a value is a valid member of a specified Enum class.
+
+    This function attempts to convert the provided `obj` into a member of the
+    `enum_class`. It tries to match by value and then by name
+    (case-insensitive). If conversion is not possible, a warning is logged, and
+    the specified `default` Enum member is returned.
+
+    Args:
+        obj (Any): The object to be ensured as an Enum member.
+        name (str): The human-readable name for the object, used in log
+            messages.
+        enum_class (Type[Enum]): The Enum class to validate against.
+        default (Enum | None): The default Enum member to return if `obj` cannot
+            be converted or is invalid. Defaults to `None`.
+        context (Dict[str, Any] | None): An optional dictionary of additional
+            context for logging.
+
+    Returns:
+        (Enum): The validated Enum member, or the `default` Enum member.
+    """
+    ctx: dict[str, Any] = {
+        **(context or {}),
+        "param_name": name,
+        "object_attempted": obj,
+        "expected_enum_class": enum_class.__name__,
+    }
+
+    if isinstance(obj, enum_class):
+        return obj
+
+    # Try to convert by value
+    try:
+        return enum_class(obj)
+    except ValueError:
+        pass  # Continue to try by name
+
+    # Try to convert by name (case-insensitive)
+    if isinstance(obj, str):
+        try:
+            return enum_class[obj.upper()]  # Enums are often uppercase
+        except KeyError:
+            pass
+
+    log_warning(
+        f"'{name}' is not a valid member of {enum_class.__name__}. "
+        f"Attempted value: {obj}. Using default.",
+        {**ctx, "_from_validation_function": True},
+    )
+    return default
+
+
 def ensure_string(
     obj: Any,
     name: str,
@@ -442,6 +469,7 @@ def ensure_string(
                 "actual_type": actual_type_name,
                 "expected_type": "str",
                 "default_object_used": default,
+                "_from_validation_function": True,
             },
         )
         return str(obj) if obj is not None else default
@@ -488,6 +516,7 @@ def ensure_non_negative_int(
                 "actual_type": actual_type_name,
                 "expected_type": "int",
                 "default_object_used": default,
+                "_from_validation_function": True,
             },
         )
         return max(0, int(obj) if isinstance(obj, (int, float)) else default)
@@ -556,6 +585,7 @@ def ensure_int_in_range(
                 "min_val": min_val,
                 "max_val": max_val,
                 "default_object_used": default,
+                "_from_validation_function": True,
             },
         )
 
@@ -634,6 +664,7 @@ def ensure_list_of_type(
                 "actual_type": actual_type_name,
                 "expected_type": "list",
                 "default_object_used": default,
+                "_from_validation_function": True,
             },
         )
         return default
@@ -661,6 +692,7 @@ def ensure_list_of_type(
                         "index": i,
                         "object_attempted": item,
                         "error_details": "Item failed validation",
+                        "_from_validation_function": True,
                     },
                 )
                 # Skip invalid items
@@ -695,6 +727,7 @@ def ensure_list_of_type(
                         "index": i,
                         "object_attempted": item,
                         "error_details": message,
+                        "_from_validation_function": True,
                     },
                 )
 
@@ -735,6 +768,7 @@ def safe_get_attribute(obj: Any, name: str, default: Any = None) -> Any:
                 "actual_type": _get_type_display_name(type(obj)),
                 "default_object_used": default,
                 "error_details": f"Missing attribute '{name}'",
+                "_from_validation_function": True,
             },
         )
         return default
